function [eigv_B_FEM,U_B_FEM] = SCR_Beam_FEM_Fun (w , z_top , H , x , s , z , zd , theta , T, mtot, EI , EA , plane , clamped , n,macc,mloc)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1)Out of plane dynamics of Steel Catenary risers (SCR)
% 2)Pure Catenary (Own weight and no currents)
% 3)FEM (2D frame element) [Bending stiffness > 0]
% 4)allow axial deformation (but neglect the effect of these deformations on
%   the variation of the effective tension)[limited extensibility]
%
% Raed Lubbad Feb./16/2009
% Last modified by Ingrid Simensen May 2015
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




nm=length(mloc);          % number of dummy elements
nel=length(x)-1;          % number of elements
nnel=2;                   % number of nodes per element
ndof=3;                   % number of dofs per node
nnode=(nnel-1)*nel+1;     % total number of nodes in system
sdof=nnode*ndof;          % total system dofs  



if clamped == 0          % % Simply supported B.C
% -------------    
bcdof(1)=1;              % horizontal deflection at node 1 is constrained
bcdof(2)=2;              % vertical deflection at node 1 is constrained
bcdof(3)=sdof-2;         % horizontal deflection the last node is constrained
bcdof(4)=sdof-1;         % vertical deflection the last node is constrained

elseif clamped ==1       % % Fixed at both ends B.C
% ----------------
bcdof(1)=1;              % horizontal deflection at node 1 is constrained
bcdof(2)=2;              % vertical deflection at node 1 is constrained
bcdof(3)=3;              % Rotation at node 1 is constrained
bcdof(4)=sdof-2;         % horizontal deflection the last node is constrained
bcdof(5)=sdof-1;         % vertical deflection the last node is constrained
bcdof(6)=sdof;           % Rotation at the last node is constrained
else
    fprintf('error: clamped must be either 1 or 0 ')
end

kk=zeros(sdof,sdof);      % initialization of system stiffness matrix
mm=zeros(sdof,sdof);      % initialization of system mass matrix
index=zeros(nel*ndof,1);  % initialization of index vector


for iel=1:nel                   % loop for the total number of elements
   
node1=iel;                      % starting node number for element 'iel'
node2=iel+1;                    % ending node number for element 'iel'

if plane == 0
 
leng = s(node2)-s(node1);
beta = pi/2;

elseif plane == 1
    
x1=x(node1); z1=z(node1);       % x and y coordinate values of 'node1'
x2=x(node2); z2=z(node2);       % x and y coordinate values of 'node2'

leng=sqrt((x2-x1)^2+(z2-z1)^2); % length of element 'iel'

if (x2-x1)==0;                  % compute the angle between the local and global axes    
   beta=pi/2; 
else
   beta=atan((z2-z1)/(x2-x1));
end 

else
    
  fprintf('error: plane must be either 1 or 0 \n') 
  
end


%T_av = (T(node1)+T(node2))/2;   % Average tension in the element 'iel'
T_e_b = T(node1);
T_e_t = T(node2);


index=feeldof1(iel,nnel,ndof);  % extract system dofs associated with element
if ismember((iel+1),mloc)
[k,m]=feframe3(EI,EA,leng,mtot,T_e_b,T_e_t,beta,macc); % compute element stiffness matrix
elseif ismember(iel,mloc)
[k,m]=feframe4(EI,EA,leng,mtot,T_e_b,T_e_t,beta,macc); % compute element stiffness matrix
else
[k,m]=feframe2(EI,EA,leng,mtot,T_e_b,T_e_t,beta); % compute element stiffness matrix
end
kk=feasmbl1(kk,k,index); % assemble element matrices into system matrix 
mm=feasmbl1(mm,m,index); % assemble element mass matrices into system matrix

end


[kn,mn]=feaplycs(kk,mm,bcdof);  % apply the boundary conditions

[V,D]=eig(kn,mn);   % solve the matrix equation and print

[lambda,k]= sort(diag(D));
V= V(:,k);
omega = sqrt(lambda);

eigv_B_FEM = omega(length(bcdof)+1:length(bcdof)+n);

for ii = 1:n
    Vj = V(:,length(bcdof)+ii);
for jj = 1:nnode
    U_B_FEM(jj,ii) = Vj((3*jj)-2);                   
end
clear Vj
end


