%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1)Matlab program for Eigenvalue analysis of Steel Catenary risers (SCR)
% 2)B.Cs. Simply supported
% 3)Pure Catenary (Own weight and no currents)
%
% 4)Case1: Beam (Bending stiffness > 0)
%          ----------------------------
%     - FEM (2D frame element) 
%     - allow axial deformation (but neglect the effect of these deformations on
%       the variation of the effective tension)[limited extensibility]
%
% Raed Lubbad Feb./13/2009
% Last modified by Ingrid Simensen 10.06.15
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc, clear all, close all


% Input
% -----
%% Masses with numbering according to thesis
% m=1; load Static_Beam_NoLoad_mass1_calc   % Gives 'w','z_top','H','x', 's','z','zd','theta','T'.
% m=2; load Static_Beam_NoLoad_mass2_calc   % Gives 'w','z_top','H','x', 's','z','zd','theta','T'.
% m=3; load Static_Beam_NoLoad_mass3_calc   % Gives 'w','z_top','H','x', 's','z','zd','theta','T'.
% m=4; load Static_Beam_NoLoad_mass4_calc   % Gives 'w','z_top','H','x', 's','z','zd','theta','T'.
% m=5; load Static_Beam_NoLoad_mass5_calc   % Gives 'w','z_top','H','x', 's','z','zd','theta','T'.
% m=6; load Static_Beam_NoLoad_massExt_calc   % Gives 'w','z_top','H','x', 's','z','zd','theta','T'.
%%No Load: 
%m=7; load Static_Beam_NoLoad_calc   % Gives 'w','z_top','H','x', 's','z','zd','theta','T'.


%Accelerometer mass locations

mloc=[34,56,84,112,140,168,196,224,252,281];

%Masses:
M=[0.00584,0.01584,0.02584,0.03584,0.04584,1,0];

mtot = 0.357 + 1000*(pi/4)*(14*10^-3)^2;         % total mass, included added mass (kg/m)[mtot=(Rs*As)+(Ri*Ai)+(Re*Ae)] % MarinTekk 2001
macc = M(m);

EI = 46.2;                                        % Bending Stiffness [N.m^2]  % MarinTekk 2001
EA = 2.01 * 10^6;                                 % Axial rigidity [N]         % MarinTekk 2001


n = 10;  % Number of modes to be calculated
j = 3;  % Plot modesshapes number: j

       

Moden    = [1:n];



%--------------------------------------------------------------------------
%  Linear FEM
[eigv_B_FEM,U_B_FEM] = SCR_Beam_FEM_Fun (w , z_top , H , x , s , z , zd , theta , T, mtot, EI , EA , 0 , 0 , n, macc,mloc);

% Plot the j Modeshape of a Beam SCR
figure
plot(s,U_B_FEM(:,j));


eigv_B_FEM_Hz=eigv_B_FEM/(2*pi());

%Save acc-position mode-shape for MAC calculations 
mloc_old=[28,56,83, 110, 138,165,220,247,275];
mloc_ex7=[34,56,84,112,140,168,224,252,281]; %removed defect accelerometer 7
if m==7
    U_BM_FEM=U_B_FEM(mloc_old,:);
else
    U_BM_FEM=U_B_FEM(mloc_ex7,:);
end
filename=['eig_U_wmass',num2str(m)];
save (filename, 'U_BM_FEM', 'eigv_B_FEM_Hz');


