

function [xi,fnat]=TDD_fnat_damp(cid,time)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Matlab function for calculation of natural frequency and damping ratio
% for TDD_method.mat
% Input: cid - vector of participation factors
%        time - vector of time instances corresponding to participation factors          
% Output: xi - damping ratio
%         fnat - natural frequency
%          ----------------------------
%
% Ingrid Simensen 10.06.2015
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


xi=[];
fn=[];
c=xcorr(cid);
%only positive lags
c=c(length(time):2*length(time)-1);

%find damped period from correlation function

[Td]=MZerocross([time',c']); %MZerocross from  Hernandez-Walls, R. (2012).MZerocross:Zerocrossing Method. A MATLAB file.   http://www.mathworks.com/matlabcentral/fileexchange/

figure(1)
plot(time,c)
title({[' Autocorrelation of modal participation']},'fontsize',18)
xlabel('Time [s] ','fontsize',18)
ylabel({['Magnitude'],[ ]},'fontsize',16)
set(gca,'fontsize',16)

mindist=Td(1)*100-Td(1)*100/16;

[Magm,Pos]=findpeaks(c,'MinPeakDistance',mindist);

figure(2);plot(time,c,time(Pos), Magm,'*')
title({[' Autocorrelation of modal participation,'],['including identified peaks']},'fontsize',18)
xlabel('Time [s] ','fontsize',18)
ylabel({['Magnitude'],[ ]},'fontsize',16)
set(gca,'fontsize',16)

%dec=zeros(1,length(Magm)/2-1);
for i=4:length(Magm)/2-1
    dec(i-3)=log((Magm(1))/(Magm(i)));
end

figure(3)
plot(4:length(Magm)/2-1,dec,'o')
title({[' Logarithmic decrement']},'fontsize',18)
xlabel('k','fontsize',18)
ylabel({['k','\delta'],[ ]},'fontsize',16)
set(gca,'fontsize',16)
num=input('Number of points in first near linear slope: ');

r=polyfit(1:num,dec(1:num),1);
xid=r(1)/sqrt(r(1)^2+(2*pi)^2);
fd=1/Td(1);
f=fd/sqrt(1-xid^2);
fnat=f;
xi=xid;
end


% matfile=['Mode_damping.mat']; % Need to convert index to string
% save (matfile, 'fnat', 'eta');