clc;
close all;
clear all;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Matlab program for OMA analysis of Steel Catenary risers (SCR) using
% the TDD method
%
% Reference:
%   "A new method to extract modal parameters using output-only responses."
%   by Kim, B. Stubbs, N. and Park,T. (2005)
%          ----------------------------
%
% Ingrid Simensen 10.06.2015
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%Choose current velocity case

%%0.12 m/s current velocity case
%load('posX12_90.mat');load('time12_90.mat');p=0.12;q='12';

%%0.24 m/s current velocity case
load('posX24_90.mat');load('time24_90.mat');p=0.24;q='24';

%%0.34m/s current velocity case
% load('posX34_90.mat');load('time34_90.mat');p=0.34;q='34';


%% Number of peaks to inspect
pn=15;%5%10%20

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Measured data
x=posX;

% Sampling frequency:
Fs = 100; 
		    
% The maximum frequency that 
% can be shown in the spectrum is 
% always half the sampling
% frequency (Nyquist frequency):
Fn = Fs/2; 
    
% Time vector,
% sampled at Fs Hz:
t = time-time(1);
% Creating empty energy matrix
E=[];

% Create PSD for peak identification
for i=1:length(x(:,1))
% figure(i)
[pxx(i,:),f]=pwelch(x(1,:),500,20,[],Fs);
%% Plot for control
% plot(f,10*log10(pxx(i,:)));
% title({[' PSD of output from sensor ',num2str(i)]},'fontsize',18)
% xlabel('Frequency [Hz]','fontsize',18)
% ylabel({[ 'Magnitude [dB]']},'fontsize',18)
% set(gca,'fontsize',16)
% %Create PSD matrix for sensors 1-6+8-10
pxx(i,:)=10*log10(pxx(i,:));
end

%Loop to investigate all sensor output peaks
for h=5%1:length(pxx(:,1))
%Empty output matrices
Uid=[];
cid=[]; 
Xfil=[];
fnat=[];
xim=[];
notf=[];
note=[];

% Identify peaks in PSD   
[Magm,Pos]=findpeaks(pxx(h,:)');
[posm,r]=sort(Pos);
Mag=Magm(r);
% Show PSD with peaks for check
figure(4)
plot(f,pxx(h,:),f(posm),Mag)
title({['PSD for accelerometer' num2str(h)],['with identified peaks']},'fontsize',18)
xlabel('Frequency [Hz]','fontsize',18)
ylabel({[ 'Magnitude [dB]']},'fontsize',18)
set(gca,'fontsize',16)

% pause on
% pause
% err=input('Are the identified peaks satisfactory? Yes/no=1/0: ');
% if err==0
%     disp('Min peak height or peak distance must be changed')
%     return
% end

peak=['Number of peaks=',num2str(length(posm(:,1)))];
disp(peak);
%p=input('Input number of peaks to investigate (must be less than number of identified peaks): ');
l=f(posm(1:pn));

%Frequency ranges
F1=l-0.1;
F2=l+0.1;

% Loop to investigate all peaks
for k=1:length(F1)
% Loop to filter all sensor output

for i=1:length(x(:,1))
% 
%normalized frequency is given by the equation 
% 
 Fnorm1 = F1(k)/Fn;  
 Fnorm2 = F2(k)/Fn;


% [b,a] = butter(n,Wn,'ftype') designs a highpass, lowpass, 
% bandpass or bandstop filter, where the string 'ftype' is 'high', 'low',
% 'bandpass' or 'stop', as described below.
[b,a] = butter(3, [Fnorm1, Fnorm2],'bandpass');

% 
% % Apply the filter
Xfiltered(i,:) = filtfilt(b,a, x(i,:));

% %Look at the filtered spectrum:
% figure(i+9)
% PlotFFT(Xfiltered(i,:),Fs);  

end
% Create energy matrix (correlation matrix)
E=Xfiltered*Xfiltered';

% Decompose E by SVD
[U,S,V]=svd(E);

% Create participation vector
c=1/(U(:,1)'*U(:,1))*U(:,1)'*Xfiltered;

% Plot spectrum of participation (should only have one peak), Identified
% mode-shape, and participation in time
figure(i+1)
subplot(3,1,1)
[cxx(i,:),fc]=pwelch(c,length(time)/2,0,length(time)/2,Fs);
plot(fc,10*log10(cxx(i,:)));
title({[' PSD of mode participation']},'fontsize',18)
xlabel('Frequency [Hz] ','fontsize',18)
ylabel({['Magnitude [dB]']},'fontsize',16)
set(gca,'fontsize',16)

MS=[0;U(:,1);0];
%Normalize
for j=1:length(MS(1,:))
ma=max(abs(MS(:,j)));
for i=1:length(MS(:,1))
    MS(i,j)=MS(i,j)/ma;
end
ma=0;
end 
yy=spline(0:10,MS,0:0.01:10);
subplot(3,1,2)
plot(0:0.01:10,yy)
title({[' Spline-fit of mode-shape']},'fontsize',18)
xlabel('Sensor point','fontsize',18)
ylabel({[ 'Normalized '],['Amplitude']},'fontsize',16)
set(gca,'fontsize',16)

subplot(3,1,3)
plot(time, c)
title({[' Time-series of participation']},'fontsize',18)
xlabel('Time [s] ','fontsize',18)
ylabel({['Magnitude']},'fontsize',16)
set(gca,'fontsize',16)
disp('Press any key')
pause
% Evaluate plot
y=input('Does the identified mode-shape and modal participation look ok?: (Yes/no=1/0): ');
%Calculate natural frequency and damping
if y==1
    cid=[cid;c];
   Uid=[Uid,MS];
   [xi,fn]=TDD_fnat_damp(c,time)
   xim=[xim,xi];
   fnat=[fnat,fn];
else
    [xi,fn]=TDD_fnat_damp(c,time)
    notf=[notf,fn];
    note=[note,xi];
end

end
% Save results to file
txtfile=['ModeParShapes_sensor' num2str(h) '.mat']; % Need to convert index to string
save (txtfile, 'xim','fnat', 'Uid', 'notf','note','cid');
disp('Press any key for plotting of mode shape')
pause
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot identified mode shapes
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


U=Uid;

% Add interpolated points at accelerometer 7
x=[];
for j=1:length(U(1,:))
    x=[x,((U(7,j))+(U(8,j)))/2];
end
U=[U(1:7,:);x;U(8:11,:)];
%Normalize
for j=1:length(U(1,:))
ma=max(abs(U(:,j)));
for i=1:length(U(:,1))
    U(i,j)=U(i,j)/ma;
end
ma=0;
end 
    
%Plot splinefitted mode shapes
for i=1:length(U(1,:))
yy=[];
yy=spline(0:11,(U(:,i)),0:0.01:11);

figure(i)
plot(yy,0:0.01:11,real(U(:,i)),0:11,'o')
title({[' Mode shape no. ' num2str(i)],[ ' identified using TDD at ' num2str(p) ' m/s flow velocity']},'fontsize',18)
xlabel('Modeshape - Normalized amplitude ','fontsize',18)
ylabel({['Accelerometer locations'],['(s/L normalized co-ordinates)']},'fontsize',18)
axis([-1.5,1.5,0,11])
filename=['TDD0_' q '_' num2str(i)];
savefig(filename)
print('-dpng',[filename '_auto'])
%print('-dpng','-r600',[filename '_600'])
end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot MAC matrix
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load eig_U_wmass7;eig7=eigv_B_FEM_Hz;UNoload=U_BM_FEM;r='FEM';

% Add boundaries

u2=zeros(1,10);
UNoload=[u2;UNoload;u2];

U=Uid;

AutoMac=zeros(length(U(1,:)),length(UNoload(1,:)));

 for k=1:length(U(1,:))
        for j=1:length(UNoload(1,:))
       AutoMac(k,j)=MAC_func(UNoload(:,j),(U(:,k)));
        end
 end

%For figure 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
map = [255,255,255;241,245,250;226,235,243;210,223,238;199,215,233;189,208,230;175,199,224;
    165,191,220;156,185,216;148,179,214;142,174,210;130,166,206;119,158,202;116,156,201;104,148,196;
    91,139,191;84,134,188;71,124,184;69,120,175;66,115,168]/255; 
W=length(U(1,:));
if W==1
    w={['TDD',q,'_1']};
elseif W==2
    w={['TDD',q,'_1'],['TDD',q,'_2']};
elseif W==3
    w={['TDD',q,'_1'],['TDD',q,'_2'],['TDD',q,'_3']};
elseif W==4
    w={['TDD',q,'_1'],['TDD',q,'_2'],['TDD',q,'_3'], ['TDD',q,'_4']};
elseif W==5
    w={['TDD',q,'_1'],['TDD',q,'_2'],['TDD',q,'_3'], ['TDD',q,'_4'],['TDD',q,'_5']};
elseif W==6
    w={['TDD',q,'_1'],['TDD',q,'_2'],['TDD',q,'_3'], ['TDD',q,'_4'],['TDD',q,'_5'],['TDD',q,'_6']};
elseif W==7
    w={['TDD',q,'_1'],['TDD',q,'_2'],['TDD',q,'_3'], ['TDD',q,'_4'],['TDD',q,'_5'],['TDD',q,'_6'],['TDD',q,'_7']};
elseif W==8
    w={['TDD',q,'_1'],['TDD',q,'_2'],['TDD',q,'_3'], ['TDD',q,'_4'],['TDD',q,'_5'],['TDD',q,'_6'],['TDD',q,'_7'],['TDD',q,'_8']};
elseif W==9
    w={['TDD',q,'_1'],['TDD',q,'_2'],['TDD',q,'_3'], ['TDD',q,'_4'],['TDD',q,'_5'],['TDD',q,'_6'],['TDD',q,'_7'],['TDD',q,'_8'],['TDD',q,'_9']};
elseif W==10
    w={['TDD',q,'_1'],['TDD',q,'_2'],['TDD',q,'_3'], ['TDD',q,'_4'],['TDD',q,'_5'],['TDD',q,'_6'],['TDD',q,'_7'],['TDD',q,'_8'],['TDD',q,'_9'],['TDD',q,'_1_0']};
else
    disp('Number of identified mode shapes outside of plottable range')
    return
end
no={'FEM_1','FEM_2','FEM_3','FEM_4','FEM_5','FEM_6','FEM_7','FEM_8','FEM_9','FEM_1_0'};
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(i+1)

mm=size(AutoMac);
colormap((map));
imagesc(AutoMac)
textStrings = num2str(AutoMac(:),'%0.2f');  %# Create strings from the matrix values
textStrings = strtrim(cellstr(textStrings));  %# Remove any space padding
[x,y] = meshgrid(1:mm(2),1:mm(1));   %# Create x and y coordinates for the strings
hStrings = text(x(:),y(:),textStrings(:),...      %# Plot the strings
                'HorizontalAlignment','center');
midValue = mean(get(gca,'CLim'));  %# Get the middle value of the color range
textColors = repmat(AutoMac(:) > midValue,1,3);  %# Choose white or black for the
                                             %#   text color of the strings so
                                             %#   they can be easily seen over
                                             %#   the background color
set(hStrings,{'Color'},num2cell(textColors,2));  %# Change the text colors
set(hStrings,'fontsize',16)
% set(gca,'XTick',1:5,...                         %# Change the axes tick marks
colorbar
title({['MAC matrix of the TDD ', num2str(p), ' m/s solutions vs ',r,' solutions']})

 
set(gca,'XTick',1:mm(2),...                         %# Change the axes tick marks
        'XTickLabel',no,...  %#   and tick labels
        'YTick',1:mm(1),...
        'YTickLabel',w,...
        'TickLength',[0 0]);
set(gca, 'fontsize',16)
set(gcf,'units','inches','OuterPosition', [0 0 12 5])
  filename=['MACMATRIX_TDD',q];
  savefig(filename)
    set(gcf,'PaperPositionMode','auto')
 print('-dpng',[filename '_auto'])
 %print('-dpng','-r600',[filename '_600'])



